;;; Copyright (c) 1996-2002 Brian D. Carlstrom

;;; Primitives
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Java Object
(define to-string (new "bdc.scheme.procedure.ToString"))
(define equals? (new "bdc.scheme.procedure.JavaEquals"))
(define list-add (new "bdc.scheme.procedure.ListAdd"))
(define list-remove (new "bdc.scheme.procedure.ListRemove"))
(define list-clear (new "bdc.scheme.procedure.ListClear"))
(define hasMoreElements (new "bdc.scheme.procedure.HasMoreElements"))
(define nextElement (new "bdc.scheme.procedure.NextElement"))
(define map-get (new "bdc.scheme.procedure.MapGet"))
(define map-put (new "bdc.scheme.procedure.MapPut"))
(define hashtable-elements (new "bdc.scheme.procedure.HashtableElements"))
(define hashtable-keys (new "bdc.scheme.procedure.HashtableKeys"))

;; Processes
(define exec (new "bdc.scheme.procedure.Exec"))
(define exit (new "bdc.scheme.procedure.Exit"))
(define wait (new "bdc.scheme.procedure.Wait"))

;; Util
(define fmt-s (new "bdc.scheme.procedure.FmtS"))


;;; Derived
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define-rewriter 'while
  (lambda (expr)
    `(until (not ,(cadr expr)) . ,(cddr expr))))

(define (system)
  (load "bdc/scheme/system.scm"))

(define (reduce cons nil list)          ;used by length, append, etc.
  (if (null? list)
      nil
      (cons (car list) (reduce cons nil (cdr list)))))

(define (any? pred l)
  (do ((l l (cdr l))
       (result '()))
      ((or (null? l)
           (boolean? result))
       (if (boolean? result) result #f))
    (cond ((pred (car l))
           (set! result #t)))))

(define (make-traced name p)
  (lambda n
    (write name) (write n)
    (apply p n)))

(define (timer thunk)
  (let ((start (new "java.util.Date")))
    (thunk)
    (let ((finish (new "java.util.Date")))
      (/ (- (get-time finish) (get-time start)) 1000))))

(define-rewriter 'time
  (lambda (expr)
    `(timer (lambda () . ,(cdr expr)))))

(define (make-string-index char=?)
  (lambda (s1 s2)
    (let ((len1 (string-length s1))
          (len2 (string-length s2)))
      (do ((i 0 (+ i 1))
           (max (- len1 len2))
           (done #f)
           (result '()))
          ((or (> i max) done) result)
        (do ((n len2 (- n 1))
             (j i (+ j 1))
             (k 0 (+ k 1))
             (innerdone #f))
            ((or (= n -1) innerdone)
             (cond ((= n -1)
                    (set! result i)
                    (set! done #t))))
          (if (or (= len1 j)
                  (= len2 k)
                  (not (char=? (string-ref s1 j)
                               (string-ref s2 k))))
              (set! innerdone #t)))))))

(define string-index (make-string-index char=?))
(define string-index-ci (make-string-index char-ci=?))

(define (make-Map)
  (new "java.util.HashMap"))

(define (justify style size str)
  (let ((result (make-string size #\space))
        (len    (string-length str))
        (start  0)
        (stop   size))
    (cond ((eq? style 'left)
           (set! stop (min size len)))
          ((eq? style 'right)
           (set! start (max 0 (- size len))))
          (else
           (log-msg "unknown style in justify %s")))
    (do ((src  0     (+ src  1))
         (dest start (+ dest 1)))
        ((= dest stop) result)
      (string-set! result dest (string-ref str src)))))

(define (run . args)
  (wait (exec (apply vector args))))

(define (vector-adjoin! v x)
  (let ((z (vector-length v)))
    (do ((i 0 (+ i 1))
         (found #f (equal x (vector-ref v i))))
        ((or found (= i z))
         (if (not found)
             (vector-addElement v x))))))

(define (number->double n)
  (+ (- n 0.5) 0.5))
